@@ -1,20 +1,18 @@
 # Calc command.
 
 import logging
-import math
-import os.path
-import re
 import sys
 import traceback
-import warnings
 
 import click
 import snuggs
+from cligj import files_inout_arg
 
 import rasterio
 from rasterio.fill import fillnodata
 from rasterio.features import sieve
-from rasterio.rio.cli import cli
+from rasterio.rio.cli import (
+    cli, dtype_opt, masked_opt, output_opt, resolve_inout)
 
 
 def get_bands(inputs, d, i=None):
@@ -37,27 +35,16 @@ def read_array(ix, subix=None, dtype=None):
 
 @cli.command(short_help="Raster data calculator.")
 @click.argument('command')
-@click.argument(
-    'files',
-    nargs=-1,
-    type=click.Path(resolve_path=False),
-    required=True,
-    metavar="INPUTS... OUTPUT")
+@files_inout_arg
+@output_opt
 @click.option('--name', multiple=True,
               help='Specify an input file with a unique short (alphas only) '
                    'name for use in commands like '
                    '"a=tests/data/RGB.byte.tif".')
-@click.option('--dtype',
-              type=click.Choice(['ubyte', 'uint8', 'uint16', 'int16', 'uint32',
-                                'int32', 'float32', 'float64']),
-              default=None,
-              help="Output data type (default: float64).")
-@click.option('--masked/--not-masked',
-              default=True,
-              help="Evaluate expressions using masked arrays (the default) "
-                   "or ordinary numpy arrays.")
+@dtype_opt
+@masked_opt
 @click.pass_context
-def calc(ctx, command, files, name, dtype, masked):
+def calc(ctx, command, files, output, name, dtype, masked):
     """A raster data calculator
 
     Evaluates an expression using input datasets and writes the result
@@ -104,10 +91,10 @@ def calc(ctx, command, files, name, dtype, masked):
 
     try:
         with rasterio.drivers(CPL_DEBUG=verbosity > 2):
-            output = files[-1]
+            output, files = resolve_inout(files=files, output=output)
 
             inputs = ([tuple(n.split('=')) for n in name] +
-                      [(None, n) for n in files[:-1]])
+                      [(None, n) for n in files])
 
             with rasterio.open(inputs[0][1]) as first:
                 kwargs = first.meta
