@@ -2,9 +2,7 @@ import logging
 import os
 import re
 import sys
-
-import click
-from click.testing import CliRunner
+import numpy
 
 import rasterio
 from rasterio.rio import features
@@ -31,6 +29,24 @@ TEST_FEATURES = """{
     "type": "Feature"
 }"""
 
+TEST_MERC_FEATURES = """{
+  "geometry": {
+      "coordinates": [
+          [
+              [-11858134, 4808920],
+              [-11868134, 4804143],
+              [-11853357, 4804143],
+              [-11840000, 4812000],
+              [-11858134, 4808920]
+          ]
+      ],
+      "type": "Polygon"
+  },
+  "properties": {
+      "val": 10
+  },
+  "type": "Feature"
+}"""
 
 # > rio shapes tests/data/shade.tif --mask --sampling 500 --projected --precision 0
 TEST_MERC_FEATURECOLLECTION = """{
@@ -78,11 +94,165 @@ TEST_MERC_FEATURECOLLECTION = """{
 }"""
 
 
-def test_err():
-    runner = CliRunner()
+def test_mask(runner, tmpdir):
+    output = str(tmpdir.join('test.tif'))
+
+    with rasterio.open('tests/data/shade.tif') as src:
+        src_data = src.read(1, masked=True)
+
+        result = runner.invoke(
+            features.mask,
+            ['tests/data/shade.tif', output, '--geojson-mask', '-'],
+            input=TEST_MERC_FEATURES
+        )
+        assert result.exit_code == 0
+        assert os.path.exists(output)
+
+        masked_count = 0
+        with rasterio.open(output) as out:
+            assert out.count == src.count
+            assert out.shape == src.shape
+            out_data = out.read(1, masked=True)
+
+            # Make sure that pixels with 0 value were converted to mask
+            masked_count = (src_data == 0).sum() - (out_data == 0).sum()
+            assert masked_count == 79743
+            assert out_data.mask.sum() - src_data.mask.sum() == masked_count
+
+        # Test using --all-touched option
+        result = runner.invoke(
+            features.mask,
+            [
+                'tests/data/shade.tif', output,
+                '--all',
+                '--geojson-mask', '-'
+            ],
+            input=TEST_MERC_FEATURES
+        )
+        assert result.exit_code == 0
+        with rasterio.open(output) as out:
+            out_data = out.read(1, masked=True)
+
+            # Make sure that more pixels with 0 value were converted to mask
+            masked_count2 = (src_data == 0).sum() - (out_data == 0).sum()
+            assert masked_count2 > 0 and masked_count > masked_count2
+            assert out_data.mask.sum() - src_data.mask.sum() == masked_count2
+
+        # Test using --invert option
+        result = runner.invoke(
+            features.mask,
+            [
+                'tests/data/shade.tif', output,
+                '--invert',
+                '--geojson-mask', '-'
+            ],
+            input=TEST_MERC_FEATURES
+        )
+        assert result.exit_code == 0
+        with rasterio.open(output) as out:
+            out_data = out.read(1, masked=True)
+            # Areas that were masked when not inverted should now be 0
+            assert (out_data == 0).sum() == masked_count
+
+    # Test with feature collection
     result = runner.invoke(
-        features.shapes, ['tests/data/shade.tif', '--bidx', '4'])
-    assert result.exit_code == 1
+        features.mask,
+        ['tests/data/shade.tif', output, '--geojson-mask', '-'],
+        input=TEST_MERC_FEATURECOLLECTION
+    )
+    assert result.exit_code == 0
+
+    # Missing GeoJSON should make copy of input to output
+    output2 = str(tmpdir.join('test2.tif'))
+    result = runner.invoke(
+        features.mask,
+        ['tests/data/shade.tif', output2]
+    )
+    assert result.exit_code == 0
+    assert os.path.exists(output2)
+    with rasterio.open('tests/data/shade.tif') as src:
+        with rasterio.open(output2) as out:
+            src_data = src.read(1, masked=True)
+            out_data = out.read(1, masked=True)
+            assert numpy.array_equal(src_data, out_data)
+
+    # Invalid JSON should fail
+    result = runner.invoke(
+        features.mask,
+        ['tests/data/shade.tif', output, '--geojson-mask', '-'],
+        input='{bogus: value}'
+    )
+    assert result.exit_code == 2
+    assert 'GeoJSON could not be read' in result.output
+
+    result = runner.invoke(
+        features.mask,
+        ['tests/data/shade.tif', output, '--geojson-mask', '-'],
+        input='{"bogus": "value"}'
+    )
+    assert result.exit_code == 2
+    assert 'Invalid GeoJSON' in result.output
+
+
+def test_mask_crop(runner, tmpdir):
+    output = str(tmpdir.join('test.tif'))
+
+    with rasterio.open('tests/data/shade.tif') as src:
+
+        result = runner.invoke(
+            features.mask,
+            [
+                'tests/data/shade.tif', output,
+                '--crop',
+                '--geojson-mask', '-'
+            ],
+            input=TEST_MERC_FEATURES
+        )
+        assert result.exit_code == 0
+        assert os.path.exists(output)
+        with rasterio.open(output) as out:
+            assert out.shape[1] == src.shape[1]
+            assert out.shape[0] < src.shape[0]
+            assert out.shape[0] == 823
+
+    # Adding invert option after crop should be ignored
+    result = runner.invoke(
+        features.mask,
+        [
+            'tests/data/shade.tif', output,
+            '--crop',
+            '--invert',
+            '--geojson-mask', '-'
+        ],
+        input=TEST_MERC_FEATURES
+    )
+    assert result.exit_code == 0
+    assert 'Invert option ignored' in result.output
+
+
+def test_mask_out_of_bounds(runner, tmpdir):
+    output = str(tmpdir.join('test.tif'))
+    # Crop with out of bounds raster should
+    result = runner.invoke(
+        features.mask,
+        ['tests/data/shade.tif', output, '--geojson-mask', '-'],
+        input=TEST_FEATURES
+    )
+    assert result.exit_code == 0
+    assert 'outside bounds' in result.output
+
+    # Crop with out of bounds raster should fail
+    result = runner.invoke(
+        features.mask,
+        [
+            'tests/data/shade.tif', output,
+            '--crop',
+            '--geojson-mask', '-'
+        ],
+        input=TEST_FEATURES
+    )
+    assert result.exit_code == 2
+    assert 'not allowed' in result.output
 
 
 def test_shapes(runner):
@@ -91,6 +261,11 @@ def test_shapes(runner):
     assert result.output.count('"FeatureCollection"') == 1
     assert result.output.count('"Feature"') == 232
 
+    # Invalid band index should fail
+    result = runner.invoke(
+        features.shapes, ['tests/data/shade.tif', '--bidx', '4'])
+    assert result.exit_code == 1
+
 
 def test_shapes_sequence(runner):
     result = runner.invoke(features.shapes, ['tests/data/shade.tif', '--sequence'])
@@ -197,7 +372,7 @@ def test_rasterize_err(tmpdir, runner):
     # Test invalid CRS value
     result = runner.invoke(features.rasterize, [output,
                                                 '--res', 1,
-                                                '--src_crs', 'BOGUS'],
+                                                '--src-crs', 'BOGUS'],
                            input=TEST_MERC_FEATURECOLLECTION)
     assert result.exit_code == 2
 
@@ -249,12 +424,12 @@ def test_rasterize(tmpdir, runner):
         assert (data == 0).sum() == 55
         assert (data == 1).sum() == 145
 
-    # Test that src_crs is written into new output
+    # Test that src-crs is written into new output
     output = str(tmpdir.join('test4.tif'))
     result = runner.invoke(features.rasterize,
                            [output,
                             '--dimensions', 20, 10,
-                            '--src_crs', 'EPSG:3857'
+                            '--src-crs', 'EPSG:3857'
                            ],
                            input=TEST_MERC_FEATURECOLLECTION)
     assert result.exit_code == 0
@@ -285,7 +460,7 @@ def test_rasterize_existing_output(tmpdir, runner):
         "type": "Feature"
     }"""
 
-    result = runner.invoke(features.rasterize, [output, '--default_value', 2],
+    result = runner.invoke(features.rasterize, [output, '--default-value', 2],
                            input=geojson)
 
     with rasterio.open(output) as out:
@@ -295,11 +470,11 @@ def test_rasterize_existing_output(tmpdir, runner):
         assert (data == 1).sum() == 125
         assert (data == 2).sum() == 20
 
-    # Confirm that a different src_crs is rejected, even if a geographic crs
+    # Confirm that a different src-crs is rejected, even if a geographic crs
     result = runner.invoke(features.rasterize,
                            [output,
                             '--res', 0.5,
-                            '--src_crs', 'EPSG:4269'
+                            '--src-crs', 'EPSG:4269'
                             ], input=TEST_FEATURES)
     assert result.exit_code == 2
 
@@ -323,12 +498,12 @@ def test_rasterize_like(tmpdir, runner):
                            [output, '--like', str(tmpdir.join('foo.tif'))], input=TEST_FEATURES)
     assert result.exit_code == 2
 
-    # Test that src_crs different than --like raster crs breaks
+    # Test that src-crs different than --like raster crs breaks
     output = str(tmpdir.join('test3.tif'))
     result = runner.invoke(features.rasterize,
                            [output,
                             '--like', 'tests/data/shade.tif',
-                            '--src_crs', 'EPSG:4326'],
+                            '--src-crs', 'EPSG:4326'],
                            input=TEST_FEATURES)
     assert result.exit_code == 2
 
@@ -340,7 +515,7 @@ def test_rasterize_property_value(tmpdir, runner):
                            [output,
                             '--res', 1000,
                             '--property', 'val',
-                            '--src_crs', 'EPSG:3857'
+                            '--src-crs', 'EPSG:3857'
                            ],
                            input=TEST_MERC_FEATURECOLLECTION)
     assert result.exit_code == 0
